# coding: utf-8

require 'vr/vruby'
require 'vr/vrcontrol'
require 'vr/vrhandler'
require 'observer'
require 'my_listbox'
require 'keycode'

class CardlistView < VRPanel
	include Observable
	include VRStdControlContainer
	include KeyCode

	TEXT = 'カードリスト'.encode('Windows-31J')

	SHIFT   = 4
  CTRL    = 8

	attr_reader :listbox

	def construct
		addControl VRStatic, 'text', TEXT, *@@text_pos
		addControl MyListbox, 'listbox', '', *@@listbox_pos
		@text.setFont @screen.factory.newfont *@@text_font
		@listbox.setFont @screen.factory.newfont *@@listbox_font
	end

	def update(command, args)
		self.send command, *args
	end

	def refresh(list)
		index = @listbox.selectedString
		tmp = if index >= 0 then @listbox.getTextOf index else nil end
		@listbox.clearStrings
		array = []
		list.each {|card| array << card.to_s}
		@listbox.setListStrings array
		@listbox.select @listbox.findString(tmp) if tmp
		@text.caption = TEXT + "(#{array.size})"
	end

	def listbox_ldblclicked(status, *args)
		index = @listbox.selectedString
		if index >= 0
			i = if (status & SHIFT) == SHIFT then @@times else 1 end
			command = if (status & CTRL) == CTRL then :add_sb else :add end
			i.times{
				self.changed
				self.notify_observers command, [@listbox.getTextOf(index).encode('UTF-8')]
			}
		end
	end

	def listbox_rdblclicked(status, *args)
		index = @listbox.selectedString
		if index >= 0
			i = if (status & SHIFT) == SHIFT then @@times else 1 end
			i.times{
				self.changed
				self.notify_observers :del, [@listbox.getTextOf(index).encode('UTF-8')]
			}
		end
	end

	def listbox_lbuttondown(status, *args)
	end

	def listbox_lbuttonup(status, *args)
	end

	def listbox_selchanged
		self.changed
		self.notify_observers :selchanged, [@listbox.getTextOf(@listbox.selectedString).encode('UTF-8')]
	end

	def listbox_char(key, code)
		case key
		when KeyCode::TAB
			step = if self.shift? then -1 else 1 end
			self.changed
			self.notify_observers :focus, [@listbox, step]
		when KeyCode::SPACE
			index = @listbox.selectedString
			self.changed
			if self.shift?
				self.notify_observers :del, [@listbox.getTextOf(index).encode('UTF-8')]
			else
				self.notify_observers :add, [@listbox.getTextOf(index).encode('UTF-8')]
			end
		when KeyCode::CTRL_Y
			self.changed
			self.notify_observers :redo, [nil]
		when KeyCode::CTRL_Z
			self.changed
			self.notify_observers :undo, [nil]
		end
	end

end
