# coding: utf-8

require 'deckbuilder_view'
require 'search_text_view'
require 'search_checkbox_view'
require 'search_range_view'
require 'search_combo_view'

require 'card'
require 'decklist_model'
require 'cardlist_model'
require 'deck_detail_model'
require 'card_detail_model'
require 'add_card_model'
require 'del_card_model'
require 'search_text_model'
require 'search_checkbox_model'
require 'search_range_model'
require 'search_combo_model'

require 'pstore'

class DeckbuilderModel
	def initialize(name, path = '.')
		# パスの設定
		@base_path = path
		@db_path = "#{@base_path}/database"
		@db_file = "#{@db_path}/#{name}.db"

		# 設定ファイルの読み込み
		require name

		# データベースの読み込み
		db = PStore.new @db_file
		db.transaction(true){ @database = db[:carddatabase][:cardlist] }

		# ウィンドウの作成
		@frm = VRLocalScreen.showForm DeckbuilderView
		@frm.add_observer self

		# デッキリスト
		@decklist_model = DecklistModel.new
		@decklist_model.add_observer @frm.decklist_view

		# カードリスト
		@cardlist_model = CardlistModel.new @database
		@cardlist_model.add_observer @frm.cardlist_view

		# デッキ詳細
		deck_detail_model = DeckDetailModel.new @database
		deck_detail_model.add_observer @frm.deck_detail_view
		@decklist_model.add_observer deck_detail_model

		# カード詳細
		card_detail_model = CardDetailModel.new @database
		card_detail_model.add_observer @frm.card_detail_view

		# カード追加
		add_card_model = AddCardModel.new
		add_card_model.add_observer @decklist_model

		# カード削除
		del_card_model = DelCardModel.new
		del_card_model.add_observer @decklist_model

		# カードリストにモジュールを結合
		@frm.cardlist_view.add_observer add_card_model
		@frm.cardlist_view.add_observer del_card_model
		@frm.cardlist_view.add_observer card_detail_model

		# デッキリストにモジュールを結合
		@frm.decklist_view.add_observer add_card_model
		@frm.decklist_view.add_observer del_card_model
		@frm.decklist_view.add_observer card_detail_model

		# 追加ボタンにカード追加を結合
		@frm.add_card_view.add_observer add_card_model
		@frm.add_x_card_view.add_observer add_card_model

		# 削除ボタンにカード削除を結合
		@frm.del_card_view.add_observer del_card_model
		@frm.del_x_card_view.add_observer del_card_model

		# クリアボタンにデッキリストを結合
		@frm.clear_deck_view.add_observer @decklist_model

		create_search_items
	end

	def update(command, args)
		self.send command, *args
	end

	def load(file)
		tmp = @decklist_model.get_deck
		begin
			db = PStore.new file
			deck = db.transaction {
				db['deck']
			}
			@decklist_model.set_deck deck
			@file_name = file
			@frm.enable_file_save
		rescue
			@decklist_model.set_deck tmp
			@frm.error_file_dialog
		end
	end

	def save(args)
		begin
			db = PStore.new @file_name
			deck = db.transaction {
				db['deck'] = @decklist_model.get_deck
			}
		rescue
			@frm.error_file_dialog
		end
	end

	def append(file)
		begin
			db = PStore.new file
			deck = db.transaction {
				db['deck'] = @decklist_model.get_deck
			}
			@file_name = file
			@frm.enable_file_save
		rescue
			@frm.error_file_dialog
		end
	end

end
