# coding: utf-8

require 'vr/vruby'
require 'observer'

require 'decklist_view'
require 'deck_detail_view'
require 'cardlist_view'
require 'card_detail_view'
require 'add_card_view'
require 'add_x_card_view'
require 'del_card_view'
require 'del_x_card_view'
require 'clear_deck_view'

class DeckbuilderView < VRForm

	include Observable
	include VRMenuUseable

	attr_reader :decklist_view
	attr_reader :deck_detail_view
	attr_reader :cardlist_view
	attr_reader :card_detail_view
	attr_reader :add_card_view
	attr_reader :add_x_card_view
	attr_reader :del_card_view
	attr_reader :del_x_card_view
	attr_reader :clear_deck_view

	M_FILE = 'ファイル(&F)'.encode('Windows-31J')

	M_FILE_LOAD 	= '開く(&O)'.encode('Windows-31J')
	M_FILE_SAVE 	= '上書き保存(&S)'.encode('Windows-31J')
	M_FILE_APPEND = '名前をつけて保存(&A)'.encode('Windows-31J')

	M_FILE_IMPORT = 'インポート(&I)'.encode('Windows-31J')
	M_FILE_EXPORT = 'エクスポート(&X)'.encode('Windows-31J')

	M_FILE_SORT_TYPE = '種類優先ソート(&T)'.encode('Windows-31J')

	M_FILE_UPDATE = 'カードデータベース更新(&U)'.encode('Windows-31J')


	M_EDIT = '編集(&E)'.encode('Windows-31J')

	M_EDIT_UNDO = "やり直し(&U)\tCtrl+Z".encode('Windows-31J')
	M_EDIT_REDO = "元に戻す(&R)\tCtrl+Y".encode('Windows-31J')

	M_EDIT_COPY = 'コピー(&C)'.encode('Windows-31J')


	M_HELP = 'ヘルプ(&H)'.encode('Windows-31J')

	M_HELP_ABOUT = "について(&A)".encode('Windows-31J')

	ENABLED  = 0
  DISABLED = VRMenuItem::DISABLED | VRMenuItem::GRAYED

	DESC = 'デッキファイル(*.dec)'.encode('Windows-31J')

  ERROR_FILE_MESSAGE      = 'ファイルアクセスに失敗しました。'.encode('Windows-31J')

  MB_OK    = 0
  MB_YESNO = 4

	def construct
		# タイトル＆ウインドウサイズ
		self.caption = @@self_title
		self.move *@@self_pos

		about_str = @@self_title + M_HELP_ABOUT

		# メニュー
		self.setMenu self.newMenu.set(
			[ [M_FILE,
					[
						[M_FILE_LOAD, 'menu_file_load'],
						[M_FILE_SAVE, 'menu_file_save', DISABLED],
						[M_FILE_APPEND, 'menu_file_append'],
						VRMenu::SEPARATOR,
						[M_FILE_IMPORT, 'menu_file_import'],
						[M_FILE_EXPORT, 'menu_file_export'],
						VRMenu::SEPARATOR,
						[M_FILE_SORT_TYPE, 'menu_file_sort_type', VRMenuItem::CHECKED],
						VRMenu::SEPARATOR,
						[M_FILE_UPDATE, 'menu_file_update']
					]
				],
				[M_EDIT,
					[
						[M_EDIT_UNDO, 'menu_edit_undo'],
						[M_EDIT_REDO, 'menu_edit_redo'],
						VRMenu::SEPARATOR,
						[M_EDIT_COPY, 'menu_edit_copy']
					]
				],
				[M_HELP,
					[
						[about_str, 'menu_help_about']
					]
				]
			]
		)

		# デッキリスト
		addControl DecklistView, 'decklist_view', '', *@@decklist_view_pos

		# カードリスト
		addControl CardlistView, 'cardlist_view', '', *@@cardlist_view_pos

		# デッキ詳細
		addControl DeckDetailView, 'deck_detail_view', '', *@@deck_detail_view_pos

		# カード詳細
		addControl CardDetailView, 'card_detail_view', '', *@@card_detail_view_pos

		# ボタン類
		addControl AddCardView, 'add_card_view', '', *@@add_card_view_pos
		addControl AddXCardView, 'add_x_card_view', '', *@@add_x_card_view_pos
		addControl DelCardView, 'del_card_view', '', *@@del_card_view_pos
		addControl DelXCardView, 'del_x_card_view', '', *@@del_x_card_view_pos
		addControl ClearDeckView, 'clear_deck_view', '', *@@clear_deck_view_pos

		create_search_items
	end

	def update(command, args)
		self.send command, *args
	end

	def menu_file_load_clicked
		file = openFilenameDialog [[DESC, '*.dec']]
		if file
			self.changed
			self.notify_observers :load, [file]
		end
	end

	def menu_file_save_clicked
		self.changed
		self.notify_observers :save, [nil]
	end

	def menu_file_append_clicked
		file = saveFilenameDialog [[DESC, '*.dec']]
		if file
			file += '.dec' if File.extname(file) != '.dec'
			self.changed
			self.notify_observers :append, [file]
		end
	end

	def enable_file_save
		@menu_file_save.state = ENABLED
	end

	def error_file_dialog
		messageBox ERROR_FILE_MESSAGE, 'Error', MB_OK
	end

end
