# coding: utf-8

require 'vr/vruby'
require 'vr/vrcontrol'
require 'vr/vrhandler'
require 'observer'
require 'my_listbox'
require 'keycode'

class DecklistView < VRPanel

	include Observable
	include VRStdControlContainer
	include KeyCode

	TEXT = 'デッキリスト'.encode('Windows-31J')

	LBUTTON = 1
	RBUTTON = 2
	SHIFT 	= 4

	attr_reader :listbox

	def construct
		addControl VRStatic, 'text', TEXT, *@@text_pos
		addControl MyListbox, 'listbox', '', *@@listbox_pos
		@text.setFont @screen.factory.newfont *@@text_font
		@listbox.setFont @screen.factory.newfont *@@listbox_font
	end

	def update(command, args)
		self.send command, *args
	end

	def refresh(deck)
		index = @listbox.selectedString
		tmp = if index >= 0 then /^\s*\d+ (.+)$/s.match(@listbox.getTextOf(index)).to_a[1] else nil end
		@listbox.clearStrings
		array = []
		num = 0
		side = true
		deck.each {|item|
			str = "#{sprintf("%2d",item[1])} #{item[0].encode('Windows-31J')}"
			if AddCardModel.sb?(item[0]) and side
				side = false
				array << "--"
			end
			array << str
			num += item[1]
			if tmp
				tmp = str if Regexp.new(tmp).match(str)
			end
		}
		@listbox.setListStrings array
		@listbox.select(@listbox.findString(tmp)) if tmp
		@text.caption = TEXT + "(#{num})"
	end

	def listbox_ldblclicked(status, *args)
		index = @listbox.selectedString
		if index >= 0
			str = /^\s*\d+ (.+)$/s.match(@listbox.getTextOf(index)).to_a[1]
			if str
				i = if status == (LBUTTON | SHIFT) then @@times else 1 end
				i.times{
					self.changed
					self.notify_observers :add, [str.encode('UTF-8')]
				}
			end
		end
	end

	def listbox_rdblclicked(status, *args)
		index = @listbox.selectedString
		if index >= 0
			str = /^\s*\d+ (.+)$/s.match(@listbox.getTextOf(index)).to_a[1]
			if str
				i = if status == (RBUTTON | SHIFT) then @@times else 1 end
				i.times{
					self.changed
					self.notify_observers :del, [str.encode('UTF-8')]
				}
			end
		end
	end

	def listbox_lbuttondown(status, *args)
	end

	def listbox_lbuttonup(status, *args)
	end

	def listbox_selchanged
		str = /^\s*\d+ (.+)$/s.match(@listbox.getTextOf(@listbox.selectedString)).to_a[1]
		if str
			self.changed
			self.notify_observers :selchanged, [str.encode('UTF-8')]
		end
	end

	def listbox_char(key, code)
		case key
		when KeyCode::TAB
			step = if self.shift? then -1 else 1 end
			self.changed
			self.notify_observers :focus, [@listbox, step]
		when KeyCode::SPACE
			index = @listbox.selectedString
			self.changed
			if self.shift?
				self.notify_observers :del, [@listbox.getTextOf(index).encode('UTF-8')]
			else
				self.notify_observers :add, [@listbox.getTextOf(index).encode('UTF-8')]
			end
		when KeyCode::CTRL_Y
			self.changed
			self.notify_observers :redo, [nil]
		when KeyCode::CTRL_Z
			self.changed
			self.notify_observers :undo, [nil]
		end
	end

end
