# coding: utf-8

require 'vr/vruby'
require 'vr/vrcontrol'
require 'observer'

require 'my_combobox'
require 'keycode'

class SearchRangeView < VRPanel

	include Observable
	include VRStdControlContainer
	include KeyCode

	FONT = 'ＭＳ ゴシック'.encode('Windows-31J')
	RANGE = '～'.encode('Windows-31J')

	def construct(
			text1_label,
			text1_pos,
			text1_font,
			first_pos,
			first_font,
			text2_pos,
			text2_font,
			last_pos,
			last_font)

		addControl VRStatic, 'text1', text1_label, *text1_pos
		addControl MyCombobox, 'first', '', *first_pos
		addControl VRStatic, 'text2', RANGE, *text2_pos
		addControl MyCombobox, 'last', '', *last_pos
		@text1.setFont @screen.factory.newfont *text1_font
		@text2.setFont @screen.factory.newfont *text2_font
		@first.setFont @screen.factory.newfont *first_font
		@last.setFont @screen.factory.newfont *last_font
	end

	def set_range(first, last, step = 1)
		@min = first / step
		@max = last / step
		@first_pos = 0
		@last_pos = @max.to_i - @min.to_i
		@step = step
		(@min .. @max).each{|i|
			@first.addString (i * step).to_s
			@last.addString (i * step).to_s
		}
		@first.select 0
		@last.select @last_pos
	end

	def first_selchanged
		first_pos = @first.selectedString
		if @first_pos != first_pos
			@first_pos = first_pos
			@last.clearStrings
			((@min + @first_pos) .. @max).each {|i|
				@last.addString (i * @step).to_s
			}
			@last.select (@last_pos - @first_pos)
			self.changed
			if @first_pos == 0 and @last_pos == (@max.to_i - @min.to_i)
				self.notify_observers :search, [nil, nil]
			else
				self.notify_observers :search, [(@min.to_i + @first_pos) * @step, (@min.to_i + @last_pos) * @step]
			end
		end
	end

  def last_selchanged
		last_pos = @last.selectedString + @first_pos
		if @last_pos != last_pos
			@last_pos = last_pos
			@first.clearStrings
			(@min .. (@min + @last_pos)).each {|i|
				@first.addString (i * @step).to_s
			}
			@first.select @first_pos
			self.changed
			if @first_pos == 0 and @last_pos == (@max.to_i - @min.to_i)
				self.notify_observers :search, [nil, nil]
			else
				self.notify_observers :search, [(@min.to_i + @first_pos) * @step, (@min.to_i + @last_pos) * @step]
			end
		end
	end

	def first_char(key, code)
		case key
		when KeyCode::TAB
			step = if self.shift? then -1 else 1 end
			self.changed
			self.notify_observers :focus, [@first, step]
		end
	end

	def last_char(key, code)
		case key
		when KeyCode::TAB
			step = if self.shift? then -1 else 1 end
			self.changed
			self.notify_observers :focus, [@last, step]
		end
	end

end
